"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.toServerFormat = exports.toClientFormat = exports.getServerResponseKeys = void 0;
const immutable_1 = require("immutable");
function getServerResponseKeys() {
    var obj = {
        subject: [],
        organisation: [],
        person: [],
        event: [],
        place: [],
        object: [],
    };
    return Object.keys(obj);
}
exports.getServerResponseKeys = getServerResponseKeys;
function toClientFormat(response) {
    var _a;
    let tags = immutable_1.OrderedMap();
    (_a = response.subject) === null || _a === void 0 ? void 0 : _a.forEach((item) => {
        const { name, description, qcode, source, altids, aliases, original_source } = item;
        const tag = {
            name,
            description,
            qcode,
            source,
            original_source,
            aliases,
            altids,
            group: {
                kind: 'scheme',
                value: item.scheme,
            },
        };
        tags = tags.set(tag.qcode, tag);
    });
    const others = [];
    if (response.organisation != null) {
        others.push({ group: 'organisation', items: response.organisation });
    }
    if (response.person != null) {
        others.push({ group: 'person', items: response.person });
    }
    if (response.event != null) {
        others.push({ group: 'event', items: response.event });
    }
    if (response.place != null) {
        others.push({ group: 'place', items: response.place });
    }
    if (response.object != null) {
        others.push({ group: 'object', items: response.object });
    }
    others.forEach(({ group, items }) => {
        items.forEach((item) => {
            const { name, description, qcode, source, altids, aliases, original_source } = item;
            const tag = {
                name,
                description,
                qcode,
                source,
                altids,
                aliases,
                original_source,
                group: {
                    kind: 'visual',
                    value: group,
                },
            };
            tags = tags.set(tag.qcode, tag);
        });
    });
    return tags;
}
exports.toClientFormat = toClientFormat;
function toServerFormat(items, superdesk) {
    const { assertNever } = superdesk.helpers;
    const result = {};
    items.forEach((item) => {
        if (item == null) {
            throw new Error('Can not be nulish.');
        }
        if (item.group.kind === 'scheme') {
            if (result.subject == null) {
                result.subject = [];
            }
            const { name, description, qcode, source, altids, aliases, original_source } = item;
            const subjectTag = {
                name,
                description,
                qcode,
                source,
                altids,
                scheme: item.group.value,
                aliases,
                original_source,
            };
            result.subject.push(subjectTag);
        }
        else if (item.group.kind === 'visual') {
            const groupValue = item.group.value;
            if (result[groupValue] == null) {
                result[groupValue] = [];
            }
            const { name, description, qcode, source, altids, aliases, original_source } = item;
            const tagBase = {
                name,
                description,
                qcode,
                source,
                altids,
                aliases,
                original_source,
            };
            result[groupValue].push(tagBase);
        }
        else {
            assertNever(item.group.kind);
        }
    });
    return result;
}
exports.toServerFormat = toServerFormat;
