"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getExistingTags = exports.createTagsPatch = void 0;
const immutable_1 = require("immutable");
const adapter_1 = require("./adapter");
const constants_1 = require("./constants");
function createTagsPatch(article, tags, superdesk) {
    const serverFormat = adapter_1.toServerFormat(tags, superdesk);
    const patch = {};
    adapter_1.getServerResponseKeys().forEach((key) => {
        let oldValues = immutable_1.OrderedMap((article[key] || []).map((_item) => [_item.qcode, _item]));
        const newValues = serverFormat[key];
        let newValuesMap = immutable_1.OrderedMap();
        const wasRemoved = (tag) => tag.source === constants_1.SOURCE_IMATRICS
            && oldValues.has(tag.qcode)
            && !newValuesMap.has(tag.qcode);
        newValues === null || newValues === void 0 ? void 0 : newValues.forEach((tag) => {
            newValuesMap = newValuesMap.set(tag.qcode, tag);
        });
        // Has to be executed even if newValuesMap is empty in order
        // for removed groups to be included in the patch.
        patch[key] = oldValues
            .merge(newValuesMap)
            .filter((tag) => wasRemoved(tag) !== true)
            .toArray();
    });
    return patch;
}
exports.createTagsPatch = createTagsPatch;
function getExistingTags(article) {
    const result = {};
    adapter_1.getServerResponseKeys().forEach((key) => {
        var _a;
        const values = ((_a = article[key]) !== null && _a !== void 0 ? _a : []).filter((tag) => tag.source === constants_1.SOURCE_IMATRICS);
        if (key === 'subject') {
            if (values.length > 0) {
                result[key] = values.map((subjectItem) => {
                    const { name, description, qcode, source, altids, scheme, aliases, original_source, } = subjectItem;
                    if (scheme == null) {
                        throw new Error('Scheme must be defined for all imatrics tags stored in subject field.');
                    }
                    const subjectTag = {
                        name,
                        description,
                        qcode,
                        source,
                        altids: altids !== null && altids !== void 0 ? altids : {},
                        scheme,
                        aliases,
                        original_source,
                    };
                    return subjectTag;
                });
            }
        }
        else if (values.length > 0) {
            result[key] = values.map((subjectItem) => {
                const { name, description, qcode, source, altids, aliases, original_source, } = subjectItem;
                const subjectTag = {
                    name,
                    description,
                    qcode,
                    source,
                    altids: altids !== null && altids !== void 0 ? altids : {},
                    aliases,
                    original_source,
                };
                return subjectTag;
            });
        }
    });
    return result;
}
exports.getExistingTags = getExistingTags;
