"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getNewItemComponent = void 0;
const React = require("react");
const auto_tagging_1 = require("./auto-tagging");
const react_1 = require("superdesk-ui-framework/react");
const groups_1 = require("./groups");
const common_1 = require("./common");
function getNewItemComponent(superdesk) {
    const { gettext } = superdesk.localization;
    const entityGroupsWithLabels = groups_1.getGroups(superdesk).filter((_, id) => auto_tagging_1.entityGroups.has(id));
    return class NewItem extends React.PureComponent {
        constructor(props) {
            super(props);
            this.state = {
                /**
                 * Requirements have been clarified to only allow entities.
                 * The functionality to support all types is already there,
                 * so I will simply disable other types for now
                 * and keep the code in case we need it later.
                 */
                type: 'entity',
                entityType: null,
                validationErrors: [],
                groupAvailableForNewItem: 'loading',
            };
        }
        componentDidMount() {
            common_1.getAutoTaggingVocabularyLabels(superdesk).then((vocabularyLabels) => {
                this.setState({
                    groupAvailableForNewItem: groups_1.getGroups(superdesk)
                        .filter((_, id) => auto_tagging_1.entityGroups.has(id) !== true)
                        .map((value) => value.singular)
                        .toOrderedMap()
                        .merge(vocabularyLabels)
                        .set('entity', gettext('Entity')),
                });
            });
        }
        render() {
            var _a, _b, _c;
            const { onChange, save, cancel } = this.props;
            const item = this.props.item;
            const { groupAvailableForNewItem } = this.state;
            if (groupAvailableForNewItem === 'loading') {
                return null;
            }
            return (React.createElement("div", { className: "sd-card auto-tagging-widget__card-absolute" },
                React.createElement("div", { className: "sd-card__header sd-card__header--white" },
                    React.createElement("div", { className: "sd-card__heading" }, gettext('Add entity'))),
                React.createElement("div", { className: "sd-card__content" },
                    this.state.validationErrors.map((error) => (React.createElement(react_1.Alert, { key: error, type: "alert", size: "small" }, error))),
                    React.createElement("div", { className: "form__row" },
                        React.createElement("div", { className: "sd-input" },
                            React.createElement("label", { className: "sd-input__label", htmlFor: "at-name" }, gettext('Name')),
                            React.createElement("input", { id: "at-name", className: "sd-input__input", type: "text", value: (_a = item.name) !== null && _a !== void 0 ? _a : '', onChange: (event) => {
                                    onChange(Object.assign(Object.assign({}, item), { name: event.target.value }));
                                } }))),
                    this.state.type !== 'entity' ? null : (React.createElement("div", { className: "form__row" },
                        React.createElement(react_1.Select, { label: gettext('Entity type'), value: (_b = this.state.entityType) !== null && _b !== void 0 ? _b : '', onChange: (value) => {
                                this.setState({ entityType: value });
                            } },
                            React.createElement(react_1.Option, null, gettext('Select type')),
                            entityGroupsWithLabels.map((g, id) => (React.createElement(react_1.Option, { key: id, value: id }, g.singular))).toArray()))),
                    React.createElement("div", { className: "form__row" },
                        React.createElement("div", { className: "sd-input" },
                            React.createElement("label", { className: "sd-input__label", htmlFor: "at-description" }, gettext('Description')),
                            React.createElement("input", { id: "at-description", className: "sd-input__input", type: "text", value: (_c = item.description) !== null && _c !== void 0 ? _c : '', onChange: (event) => {
                                    onChange(Object.assign(Object.assign({}, item), { description: event.target.value }));
                                } })))),
                React.createElement("div", { className: "sd-card__footer" },
                    React.createElement("button", { className: "btn sd-flex-grow", onClick: () => cancel() }, gettext('Cancel')),
                    React.createElement("button", { className: "btn btn--primary sd-flex-grow", onClick: () => {
                            var _a, _b, _c;
                            const validationErrors = [];
                            if (((_b = (_a = item.name) === null || _a === void 0 ? void 0 : _a.trim().length) !== null && _b !== void 0 ? _b : 0) < 1) {
                                validationErrors.push(gettext('Name is required.'));
                            }
                            if (this.state.type == null) {
                                validationErrors.push(gettext('Type is required.'));
                            }
                            if (this.state.type === 'entity' && this.state.entityType == null) {
                                validationErrors.push(gettext('Entity type is required.'));
                            }
                            if (validationErrors.length > 0) {
                                this.setState({ validationErrors });
                            }
                            else {
                                const groupValue = (_c = this.state.entityType) !== null && _c !== void 0 ? _c : this.state.type;
                                if (groupValue == null) {
                                    throw new Error('Should have already been validated.');
                                }
                                save(Object.assign(Object.assign({}, item), { group: {
                                        kind: 'visual',
                                        value: groupValue,
                                    } }));
                            }
                        } }, gettext('Add')))));
        }
    };
}
exports.getNewItemComponent = getNewItemComponent;
