"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDateTimeField = void 0;
const React = require("react");
const set_1 = require("date-fns/set");
const format_1 = require("date-fns/format");
const addMinutes_1 = require("date-fns/addMinutes");
const react_1 = require("superdesk-ui-framework/react");
function getDateTimeField(superdesk) {
    const { gettext, gettextPlural } = superdesk.localization;
    const { getLocaleForDatePicker } = superdesk.ui.framework;
    const { Spacer } = superdesk.components;
    const { dateToServerString } = superdesk.utilities;
    return class DateTimeField extends React.PureComponent {
        render() {
            var _a, _b, _c;
            const checkbox = (React.createElement(react_1.Switch, { value: this.props.value != null, onChange: (value) => {
                    if (value) {
                        this.props.setValue(dateToServerString(addMinutes_1.default(new Date(), this.props.config.initial_offset_minutes)));
                    }
                    else {
                        this.props.setValue(null);
                    }
                } }));
            if (this.props.value == null) {
                return (React.createElement("div", null, checkbox));
            }
            else {
                const date = new Date(this.props.value);
                const day = format_1.default(date, 'yyyy-MM-dd'); // ISO8601
                const hour = format_1.default(date, 'HH:mm'); // ISO8601
                const steps = (_b = (_a = this.props.config) === null || _a === void 0 ? void 0 : _a.increment_steps) !== null && _b !== void 0 ? _b : [];
                // Get the DatePicker locale using the language of this item
                const language = (_c = this.props.item.language) !== null && _c !== void 0 ? _c : superdesk.instance.config.default_language;
                const datePickerLocale = getLocaleForDatePicker(language);
                return (React.createElement(Spacer, { type: "horizontal", align: "center", spacing: "medium" },
                    checkbox,
                    React.createElement(Spacer, { type: "horizontal", align: "stretch", spacing: "medium" },
                        React.createElement(react_1.DatePickerISO, { dateFormat: superdesk.instance.config.view.dateformat, locale: datePickerLocale, value: day, onChange: (dateString) => {
                                if (dateString === '') {
                                    this.props.setValue(null);
                                    return;
                                }
                                const [yearStr, monthStr, dayStr] = dateString.split('-');
                                this.props.setValue(dateToServerString(set_1.default(date, {
                                    year: parseInt(yearStr, 10),
                                    month: parseInt(monthStr, 10) - 1,
                                    date: parseInt(dayStr, 10),
                                })));
                            } }),
                        React.createElement("div", { style: { display: 'flex', alignItems: 'center', height: '100%' } },
                            React.createElement("span", null, "@")),
                        React.createElement(react_1.TimePicker, { required // because it's a part of the date-time
                            : true, value: hour, onChange: (value) => {
                                const [hours, minutes] = value.split(':');
                                this.props.setValue(dateToServerString(set_1.default(date, {
                                    hours: parseInt(hours, 10),
                                    minutes: parseInt(minutes, 10),
                                })));
                            } })),
                    steps.length < 1
                        ? null
                        : (React.createElement("div", null, steps.map((step, i) => {
                            const stepAbsolute = Math.abs(step);
                            const fullHours = Math.floor(stepAbsolute / 60);
                            const remainingMinutes = stepAbsolute % 60;
                            let buttonText = '';
                            if (step >= 0) {
                                buttonText += '+';
                            }
                            else {
                                buttonText += '-';
                            }
                            if (fullHours > 0) {
                                buttonText += ' ' + gettextPlural(fullHours, '{{x}} hour', '{{x}} hours', { x: fullHours });
                            }
                            if (remainingMinutes > 0) {
                                buttonText += ' ' + gettext('{{x}} min', { x: remainingMinutes });
                            }
                            return (React.createElement(react_1.Button, { key: i, disabled: date == null, onClick: () => {
                                    if (date != null) {
                                        this.props.setValue(dateToServerString(addMinutes_1.default(date, step)));
                                    }
                                }, text: buttonText, style: "hollow", size: "small" }));
                        })))));
            }
        }
    };
}
exports.getDateTimeField = getDateTimeField;
