"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.markForUserAndSendToNextStage = exports.updateMarkedUser = exports.canChangeMarkedUser = void 0;
function canChangeMarkedUser(superdesk, article) {
    const { isPersonal, isLockedInOtherSession, isArchived, isPublished } = superdesk.entities.article;
    const { hasPrivilege } = superdesk.privileges;
    if (!hasPrivilege('mark_for_user')
        || isPersonal(article)
        || isArchived(article)
        || isPublished(article)
        || isLockedInOtherSession(article)
        || article.state === 'spiked') {
        return false;
    }
    else {
        return true;
    }
}
exports.canChangeMarkedUser = canChangeMarkedUser;
function canSendToDesk(superdesk, article) {
    var _a;
    const { isLocked } = superdesk.entities.article;
    return !isLocked(article) && ((_a = article.task) === null || _a === void 0 ? void 0 : _a.desk) != null;
}
function updateMarkedUser(superdesk, article, patch) {
    superdesk.entities.article.patch(article, patch, { patchDirectlyAndOverwriteAuthoringValues: true });
}
exports.updateMarkedUser = updateMarkedUser;
function markForUserAndSendToNextStage(superdesk, article, selectedUserId) {
    const { logger } = superdesk.utilities;
    if (!canChangeMarkedUser(superdesk, article) || !canSendToDesk(superdesk, article)) {
        logger.warn('Can not mark and send', article);
        return;
    }
    const getTask = () => {
        const task = article.task || null;
        if (task != null && task.desk != null) {
            return superdesk.entities.desk.getStagesOrdered(task.desk)
                .then((stages) => {
                const currentStageIndex = stages.findIndex((stage) => task.stage === stage._id);
                if (currentStageIndex !== -1) {
                    const nextStageIndex = currentStageIndex + 1 === stages.length ? 0 : currentStageIndex + 1;
                    return Object.assign(Object.assign({}, task), { stage: stages[nextStageIndex]._id });
                }
                return task;
            });
        }
        return Promise.resolve(task);
    };
    return getTask().then((task) => {
        updateMarkedUser(superdesk, article, { marked_for_user: selectedUserId, task });
    });
}
exports.markForUserAndSendToNextStage = markForUserAndSendToNextStage;
