"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const show_marked_user_1 = require("./show-marked-user");
const get_article_actions_1 = require("./get-article-actions");
const get_article_actions_bulk_1 = require("./get-article-actions-bulk");
const get_marked_for_me_component_1 = require("./get-marked-for-me-component");
const get_article_queries_1 = require("./get-article-queries");
const extension = {
    id: 'markForUser',
    exposes: {
        getQueryNotMarkedForAnyoneOrMarkedForMe: get_article_queries_1.getQueryNotMarkedForAnyoneOrMarkedForMe,
        getQueryMarkedForUser: get_article_queries_1.getQueryMarkedForUser,
    },
    activate: (superdesk) => {
        const { gettext } = superdesk.localization;
        return superdesk.session.getCurrentUser().then((user) => {
            const personalSpaceSections = [
                {
                    id: 'markedForMe',
                    label: gettext('Marked for me'),
                    query: {
                        term: {
                            marked_for_user: user._id,
                        },
                    },
                },
            ];
            const result = {
                contributions: {
                    globalMenuHorizontal: [get_marked_for_me_component_1.getMarkedForMeComponent(superdesk)],
                    articleListItemWidgets: [show_marked_user_1.getDisplayMarkedUserComponent(superdesk)],
                    authoringTopbarWidgets: [show_marked_user_1.getDisplayMarkedUserComponent(superdesk)],
                    personalSpace: {
                        getSections: () => personalSpaceSections,
                    },
                    notifications: {
                        'item:marked': (notification) => {
                            return {
                                body: notification.message,
                                actions: [
                                    {
                                        label: gettext('open item'),
                                        onClick: () => superdesk.ui.article.view(notification.item),
                                    },
                                ],
                            };
                        },
                        'item:unmarked': (notification) => {
                            return {
                                body: notification.message,
                                actions: [
                                    {
                                        label: gettext('open item'),
                                        onClick: () => superdesk.ui.article.view(notification.item),
                                    },
                                ],
                            };
                        },
                    },
                    entities: {
                        article: {
                            getActions: get_article_actions_1.getActionsInitialize(superdesk),
                            getActionsBulk: get_article_actions_bulk_1.getActionsBulkInitialize(superdesk),
                        },
                    },
                    monitoring: {
                        getFilteringButtons: (deskId) => {
                            const items = [
                                {
                                    label: gettext('Marked for me'),
                                    query: {
                                        marked_for_user: [user._id],
                                        'task.desk': [deskId],
                                    },
                                    displayOptions: {
                                        ignoreMatchesInSavedSearchMonitoringGroups: true,
                                    },
                                },
                            ];
                            return items;
                        },
                    },
                },
            };
            return Promise.resolve(result);
        });
    },
};
exports.default = extension;
