"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getActionsBulkInitialize = void 0;
const lodash_1 = require("lodash");
const get_mark_for_user_modal_1 = require("./get-mark-for-user-modal");
const common_1 = require("./common");
function getActionsBulkInitialize(superdesk) {
    const { gettext } = superdesk.localization;
    const { isLocked, isLockedInOtherSession } = superdesk.entities.article;
    return function getActionsBulk(articles) {
        const someItemsLocked = articles.some(isLocked);
        const someItemsLockedInOtherSession = articles.some(isLockedInOtherSession);
        if (articles.some((article) => !common_1.canChangeMarkedUser(superdesk, article))) {
            return Promise.resolve([]);
        }
        const selectedUserIds = lodash_1.uniq(articles
            .map((item) => item.marked_for_user)
            .filter((marked_for_user) => marked_for_user != null));
        const message = selectedUserIds.length > 1
            ? gettext('Items are marked for different users')
            : undefined;
        return Promise.resolve([{
                label: gettext('Mark for user'),
                icon: 'icon-assign',
                onTrigger: () => {
                    var _a;
                    superdesk.ui.showModal(get_mark_for_user_modal_1.getMarkForUserModal({
                        superdesk: superdesk,
                        markForUser: (selectedUserId) => {
                            articles.forEach((article) => {
                                common_1.updateMarkedUser(superdesk, article, { marked_for_user: selectedUserId });
                            });
                        },
                        markForUserAndSend: (selectedUserId) => {
                            articles.forEach((article) => {
                                common_1.markForUserAndSendToNextStage(superdesk, article, selectedUserId);
                            });
                        },
                        locked: someItemsLocked,
                        lockedInOtherSession: someItemsLockedInOtherSession,
                        markedForUserInitial: (_a = selectedUserIds[0]) !== null && _a !== void 0 ? _a : undefined,
                        message: message,
                    }));
                },
            }]);
    };
}
exports.getActionsBulkInitialize = getActionsBulkInitialize;
