"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getMarkForUserModal = void 0;
const React = require("react");
function getMarkForUserModal(options) {
    const { superdesk, markForUser, markForUserAndSend, locked, lockedInOtherSession, markedForUserInitial, message, } = options;
    const { gettext } = superdesk.localization;
    const { Modal, ModalHeader, ModalBody, ModalFooter, SelectUser, } = superdesk.components;
    const { logger } = superdesk.utilities;
    return class MarkForUserModalComponent extends React.Component {
        constructor(props) {
            super(props);
            this.state = {
                selectedUserId: markedForUserInitial,
            };
        }
        render() {
            return (React.createElement(Modal, { "data-test-id": "mark-for-user-modal", size: "large" },
                React.createElement(ModalHeader, { onClose: this.props.closeModal }, gettext('Mark for user')),
                React.createElement(ModalBody, null,
                    message == null ? null : (React.createElement("div", { className: "sd-alert sd-alert--hollow sd-alert--primary sd-alert--small" }, message)),
                    lockedInOtherSession === true ? (React.createElement("div", { className: "sd-alert sd-alert--hollow sd-alert--primary sd-alert--small" }, gettext('Item is locked and marked user can not be changed.'))) : null,
                    React.createElement(SelectUser, { disabled: lockedInOtherSession, onSelect: (selectedUser) => this.setState({ selectedUserId: selectedUser._id }), selectedUserId: this.state.selectedUserId, autoFocus: true })),
                React.createElement(ModalFooter, null,
                    React.createElement("button", { className: "btn", onClick: () => this.props.closeModal(), "data-test-id": "cancel" }, gettext('Cancel')),
                    markedForUserInitial !== undefined ? (React.createElement("button", { className: "btn btn--warning", disabled: lockedInOtherSession, onClick: () => {
                            this.props.closeModal();
                            markForUser(null);
                        }, "data-test-id": "unmark" }, gettext('Unmark'))) : null,
                    React.createElement("button", { className: "btn btn--primary", disabled: this.state.selectedUserId === undefined // no user selected
                            || this.state.selectedUserId === markedForUserInitial // user hasn't changed
                            || lockedInOtherSession, onClick: () => {
                            this.props.closeModal();
                            if (this.state.selectedUserId !== undefined) {
                                markForUser(this.state.selectedUserId);
                            }
                            else {
                                logger.error(new Error('selectedUserId can not be undefined'));
                            }
                        }, "data-test-id": "confirm" }, gettext('Mark for user')),
                    React.createElement("button", { className: "btn btn--primary", disabled: this.state.selectedUserId === undefined // no user selected
                            || this.state.selectedUserId === markedForUserInitial // user hasn't changed
                            || locked // can't send to another stage even if locked by current user
                        , onClick: () => {
                            this.props.closeModal();
                            if (this.state.selectedUserId !== undefined) {
                                markForUserAndSend(this.state.selectedUserId);
                            }
                            else {
                                logger.error(new Error('selectedUserId can not be undefined'));
                            }
                        }, "data-test-id": "mark-and-send" }, gettext('Mark and send')))));
        }
    };
}
exports.getMarkForUserModal = getMarkForUserModal;
