"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getSetsSync = exports.unlockAsset = exports.lockAsset = exports.updateAsset = exports.deleteAsset = exports.getAssetsCompressedBinary = exports.getAssetBinary = exports.showUploadAssetModal = exports.updateAssetMetadata = exports.getAssetsByIds = exports.getAssetById = exports.getAssetsCount = exports.setAssetSearchUrlParams = exports.getAssetSearchUrlParams = exports.queryAssets = exports.uploadAsset = void 0;
var interfaces_1 = require("../interfaces");
var apis_1 = require("../apis");
// Redux Actions & Selectors
var store_1 = require("../store");
var actions_1 = require("../store/sets/actions");
var selectors_1 = require("../store/sets/selectors");
// Utils
var common_1 = require("./common");
var api_1 = require("../utils/api");
// UI
var ui_1 = require("../utils/ui");
var uploadAssetModal_1 = require("../components/assets/uploadAssetModal");
var RESOURCE = 'sams/assets';
var COUNT_RESOURCE = RESOURCE + "/counts/";
var BINARY_RESOURCE = RESOURCE + "/binary/";
var COMPRESSED_BINARY_RESOURCE = RESOURCE + "/compressed_binary/";
var LOCK_ASSET = RESOURCE + "/lock";
var UNLOCK_ASSET = RESOURCE + "/unlock";
function uploadAsset(data, onProgress) {
    var gettext = apis_1.superdeskApi.localization.gettext;
    var notify = apis_1.superdeskApi.ui.notify;
    return apis_1.superdeskApi.dataApi.uploadFileWithProgress('/' + RESOURCE, data, onProgress)
        .catch(function (error) {
        if (api_1.isSamsApiError(error)) {
            notify.error(api_1.getApiErrorMessage(error));
        }
        else {
            notify.error(gettext('Failed to upload file.'));
        }
        return Promise.reject(error);
    });
}
exports.uploadAsset = uploadAsset;
var GRID_PAGE_SIZE = 25;
var LIST_PAGE_SIZE = 50;
function querySearchString(source, params) {
    if (params.textSearch != null && params.textSearch.length > 0) {
        source.query.bool.must.push(apis_1.superdeskApi.elasticsearch.queryString({
            query: params.textSearch,
            lenient: true,
            default_operator: 'AND',
        }));
    }
}
function querySetId(source, params) {
    if (params.setId != null && params.setId.length > 0) {
        source.query.bool.must.push(apis_1.superdeskApi.elasticsearch.term({
            field: 'set_id',
            value: params.setId,
        }));
    }
}
function querySetIds(source, params) {
    if (params.setIds != null && params.setIds.length > 0) {
        source.query.bool.must.push(apis_1.superdeskApi.elasticsearch.terms({
            field: 'set_id',
            value: params.setIds,
        }));
    }
}
function queryState(source, params) {
    if (params.state != null && params.state.length > 0) {
        source.query.bool.must.push(apis_1.superdeskApi.elasticsearch.term({
            field: 'state',
            value: params.state,
        }));
    }
}
function queryStates(source, params) {
    if (params.states != null && params.states.length > 0) {
        source.query.bool.must.push(apis_1.superdeskApi.elasticsearch.terms({
            field: 'state',
            value: params.states,
        }));
    }
}
function queryName(source, params) {
    if (params.name != null && params.name.length > 0) {
        source.query.bool.must.push(apis_1.superdeskApi.elasticsearch.queryString({
            query: "name:(" + params.name + ")",
            lenient: true,
            default_operator: 'AND',
        }));
    }
}
function queryFilename(source, params) {
    if (params.filename != null && params.filename.length > 0) {
        source.query.bool.must.push(apis_1.superdeskApi.elasticsearch.queryString({
            query: "filename:(" + params.filename + ")",
            lenient: true,
            default_operator: 'AND',
        }));
    }
}
function queryDescription(source, params) {
    if (params.description != null && params.description.length > 0) {
        source.query.bool.must.push(apis_1.superdeskApi.elasticsearch.queryString({
            query: "description:(" + params.description + ")",
            lenient: false,
            default_operator: 'AND',
        }));
    }
}
function queryMimetypes(source, params) {
    if (params.mimetypes === interfaces_1.ASSET_TYPE_FILTER.DOCUMENTS) {
        source.query.bool.must_not.push(apis_1.superdeskApi.elasticsearch.queryString({
            query: 'mimetype:(image\\/*) OR mimetype:(video\\/*) OR mimetype:(audio\\/*)',
            lenient: true,
            default_operator: 'OR',
        }));
    }
    else if (params.mimetypes !== interfaces_1.ASSET_TYPE_FILTER.ALL) {
        var typeString = void 0;
        switch (params.mimetypes) {
            case interfaces_1.ASSET_TYPE_FILTER.IMAGES:
                typeString = 'image';
                break;
            case interfaces_1.ASSET_TYPE_FILTER.VIDEOS:
                typeString = 'video';
                break;
            case interfaces_1.ASSET_TYPE_FILTER.AUDIO:
                typeString = 'audio';
                break;
        }
        if (typeString != null) {
            source.query.bool.must.push(apis_1.superdeskApi.elasticsearch.queryString({
                query: "mimetype:(" + typeString + "\\/*)",
                lenient: true,
                default_operator: 'OR',
            }));
        }
    }
}
function queryDateRange(source, params) {
    if (params.dateFrom != null || params.dateTo != null) {
        var args = { field: '_updated' };
        if (params.dateFrom != null) {
            args.gte = params.dateFrom.toISOString();
        }
        if (params.dateTo != null) {
            args.lte = params.dateTo.toISOString();
        }
        source.query.bool.must.push(apis_1.superdeskApi.elasticsearch.range(args));
    }
}
function querySizeRange(source, params) {
    if (params.sizeFrom != null || params.sizeTo != null) {
        var args = { field: 'length' };
        if (params.sizeFrom != null) {
            args.gte = params.sizeFrom * 1048576; // MB -> bytes
        }
        if (params.sizeTo != null) {
            args.lte = params.sizeTo * 1048576; // MB -> bytes
        }
        source.query.bool.must.push(apis_1.superdeskApi.elasticsearch.range(args));
    }
}
function queryAssets(params, listStyle) {
    var gettext = apis_1.superdeskApi.localization.gettext;
    var notify = apis_1.superdeskApi.ui.notify;
    var pageSize = listStyle === interfaces_1.ASSET_LIST_STYLE.GRID ?
        GRID_PAGE_SIZE :
        LIST_PAGE_SIZE;
    var source = {
        query: {
            bool: {
                must: [],
                must_not: [],
            },
        },
        size: pageSize,
        from: (params.page - 1) * pageSize,
    };
    [
        querySearchString,
        querySetId,
        querySetIds,
        queryName,
        queryDescription,
        queryFilename,
        queryMimetypes,
        queryState,
        queryStates,
        queryDateRange,
        querySizeRange,
    ].forEach(function (func) { return func(source, params); });
    if (source.query.bool.must.length === 0) {
        delete source.query.bool.must;
    }
    if (source.query.bool.must_not.length === 0) {
        delete source.query.bool.must_not;
    }
    if (Object.keys(source.query.bool).length === 0) {
        delete source.query;
    }
    var sortOrder = params.sortOrder === interfaces_1.SORT_ORDER.ASCENDING ? 1 : 0;
    var sort = "[(\"" + params.sortField + "\"," + sortOrder + ")]";
    return apis_1.superdeskApi.dataApi.queryRawJson(RESOURCE, {
        source: JSON.stringify(source),
        sort: sort,
    })
        .then(common_1.fixItemResponseVersionDates)
        .catch(function (error) {
        if (api_1.isSamsApiError(error)) {
            notify.error(api_1.getApiErrorMessage(error));
        }
        else {
            notify.error(gettext('Failed to query Assets'));
        }
        return Promise.reject(error);
    });
}
exports.queryAssets = queryAssets;
function getAssetSearchUrlParams() {
    var urlParams = apis_1.superdeskApi.browser.location.urlParams;
    var filterUndefined = apis_1.superdeskApi.helpers.filterUndefined;
    return filterUndefined({
        textSearch: urlParams.getString('textSearch'),
        setId: urlParams.getString('setId'),
        name: urlParams.getString('name'),
        description: urlParams.getString('description'),
        state: urlParams.getString('state'),
        filename: urlParams.getString('filename'),
        mimetypes: urlParams.getString('mimetypes', interfaces_1.ASSET_TYPE_FILTER.ALL),
        dateFrom: urlParams.getDate('dateFrom'),
        dateTo: urlParams.getDate('dateTo'),
        sizeFrom: urlParams.getNumber('sizeFrom'),
        sizeTo: urlParams.getNumber('sizeTo'),
        sortField: urlParams.getString('sortField', interfaces_1.ASSET_SORT_FIELD.NAME),
        sortOrder: urlParams.getString('sortOrder', interfaces_1.SORT_ORDER.ASCENDING),
    });
}
exports.getAssetSearchUrlParams = getAssetSearchUrlParams;
function setAssetSearchUrlParams(params) {
    var urlParams = apis_1.superdeskApi.browser.location.urlParams;
    urlParams.setString('textSearch', params.textSearch);
    urlParams.setString('setId', params.setId);
    urlParams.setString('name', params.name);
    urlParams.setString('description', params.description);
    urlParams.setString('state', params.state);
    urlParams.setString('filename', params.filename);
    urlParams.setString('mimetypes', params.mimetypes);
    urlParams.setDate('dateFrom', params.dateFrom);
    urlParams.setDate('dateTo', params.dateTo);
    urlParams.setNumber('sizeFrom', params.sizeFrom);
    urlParams.setNumber('sizeTo', params.sizeTo);
    urlParams.setString('sortField', params.sortField);
    urlParams.setString('sortOrder', params.sortOrder);
}
exports.setAssetSearchUrlParams = setAssetSearchUrlParams;
function getAssetsCount(set_ids) {
    var gettext = apis_1.superdeskApi.localization.gettext;
    var notify = apis_1.superdeskApi.ui.notify;
    return apis_1.superdeskApi.dataApi.queryRawJson(COUNT_RESOURCE + JSON.stringify(set_ids))
        .catch(function (error) {
        if (api_1.isSamsApiError(error)) {
            notify.error(api_1.getApiErrorMessage(error));
        }
        else {
            notify.error(gettext('Failed to get assets counts for sets'));
        }
        return Promise.reject(error);
    });
}
exports.getAssetsCount = getAssetsCount;
function getAssetById(assetId) {
    var gettext = apis_1.superdeskApi.localization.gettext;
    var notify = apis_1.superdeskApi.ui.notify;
    return apis_1.superdeskApi.dataApi.findOne(RESOURCE, assetId)
        .then(common_1.fixItemVersionDates)
        .catch(function (error) {
        if (api_1.isSamsApiError(error)) {
            notify.error(api_1.getApiErrorMessage(error));
        }
        else {
            notify.error(gettext("Failed to get asset \"" + assetId + "\""));
        }
        return Promise.reject(error);
    });
}
exports.getAssetById = getAssetById;
function getAssetsByIds(ids) {
    var gettext = apis_1.superdeskApi.localization.gettext;
    var notify = apis_1.superdeskApi.ui.notify;
    return apis_1.superdeskApi.dataApi.queryRawJson(RESOURCE, { source: JSON.stringify({
            query: {
                bool: {
                    must: [
                        apis_1.superdeskApi.elasticsearch.terms({
                            field: '_id',
                            value: ids,
                        }),
                    ],
                },
            },
        }) })
        .then(common_1.fixItemResponseVersionDates)
        .catch(function (error) {
        if (api_1.isSamsApiError(error)) {
            notify.error(api_1.getApiErrorMessage(error));
        }
        else {
            notify.error(gettext('Failed to get Assets'));
        }
        return Promise.reject(error);
    });
}
exports.getAssetsByIds = getAssetsByIds;
function updateAssetMetadata(originalAsset, originalAttachment, updates) {
    var gettext = apis_1.superdeskApi.localization.gettext;
    var notify = apis_1.superdeskApi.ui.notify;
    return Promise.all([
        apis_1.superdeskApi.entities.attachment.save(originalAttachment, __assign(__assign({}, originalAttachment), { title: updates.name, description: updates.description, internal: updates.state !== interfaces_1.ASSET_STATE.PUBLIC })),
        apis_1.superdeskApi.dataApi.patch(RESOURCE, originalAsset, updates)
            .catch(function (error) {
            if (api_1.isSamsApiError(error)) {
                notify.error(api_1.getApiErrorMessage(error));
            }
            else {
                notify.error(gettext('Failed to update asset.'));
            }
            return Promise.reject(error);
        }),
    ]);
}
exports.updateAssetMetadata = updateAssetMetadata;
function showUploadAssetModal(props) {
    var gettext = apis_1.superdeskApi.localization.gettext;
    var notify = apis_1.superdeskApi.ui.notify;
    var store = store_1.getStore();
    if (store === undefined) {
        throw new Error('SAMS store has not been initialised');
    }
    // (re)load all the Sets into the Redux store
    store.dispatch(actions_1.loadSets())
        .then(function (sets) {
        // Check if there are any usable Sets found
        // Otherwise notify the user to enable one first
        if (sets.filter(function (set) { return set.state === interfaces_1.SET_STATE.USABLE; }).length === 0) {
            notify.error(gettext('No usable Sets found. Enable one first'));
            return;
        }
        // Finally show the upload asset modal
        ui_1.showModalConnectedToStore(uploadAssetModal_1.UploadAssetModal, __assign({ dark: true, modalSize: 'fill' }, props !== null && props !== void 0 ? props : {}));
    });
}
exports.showUploadAssetModal = showUploadAssetModal;
function getAssetBinary(asset) {
    var gettext = apis_1.superdeskApi.localization.gettext;
    var notify = apis_1.superdeskApi.ui.notify;
    var downloadBlob = apis_1.superdeskApi.utilities.downloadBlob;
    return apis_1.superdeskApi.dataApi.queryRaw(BINARY_RESOURCE + asset._id)
        .then(function (res) { return res.arrayBuffer()
        .then(function (blob) {
        downloadBlob(blob, res.headers.get('Content-type'), asset.filename);
    }); })
        .catch(function (error) {
        if (api_1.isSamsApiError(error)) {
            notify.error(api_1.getApiErrorMessage(error));
        }
        else {
            notify.error(gettext('Failed to get binary for asset'));
        }
        return Promise.reject(error);
    });
}
exports.getAssetBinary = getAssetBinary;
function getAssetsCompressedBinary(asset_ids) {
    var gettext = apis_1.superdeskApi.localization.gettext;
    var notify = apis_1.superdeskApi.ui.notify;
    var downloadBlob = apis_1.superdeskApi.utilities.downloadBlob;
    return apis_1.superdeskApi.dataApi.queryRaw(COMPRESSED_BINARY_RESOURCE + JSON.stringify(asset_ids))
        .then(function (res) { return res.arrayBuffer()
        .then(function (blob) {
        downloadBlob(blob, 'application/zip', 'download');
    }); })
        .catch(function (error) {
        if (api_1.isSamsApiError(error)) {
            notify.error(api_1.getApiErrorMessage(error));
        }
        else {
            notify.error(gettext('Failed to get compressed binaries for assets'));
        }
        return Promise.reject(error);
    });
}
exports.getAssetsCompressedBinary = getAssetsCompressedBinary;
function deleteAsset(item) {
    var gettext = apis_1.superdeskApi.localization.gettext;
    var notify = apis_1.superdeskApi.ui.notify;
    return apis_1.superdeskApi.dataApi.delete(RESOURCE, item)
        .then(function () {
        notify.success(gettext('Asset deleted successfully'));
        return Promise.resolve();
    })
        .catch(function (error) {
        if (api_1.isSamsApiError(error)) {
            notify.error(api_1.getApiErrorMessage(error));
        }
        else {
            notify.error(gettext('Failed to delete the Asset'));
        }
        return Promise.reject(error);
    });
}
exports.deleteAsset = deleteAsset;
function updateAsset(original, updates) {
    var gettext = apis_1.superdeskApi.localization.gettext;
    var notify = apis_1.superdeskApi.ui.notify;
    return apis_1.superdeskApi.dataApi.patch(RESOURCE, original, updates)
        .then(function (asset) {
        notify.success(gettext('Asset updated successfully'));
        return asset;
    })
        .catch(function (error) {
        if (api_1.isSamsApiError(error)) {
            notify.error(api_1.getApiErrorMessage(error));
        }
        else {
            notify.error(gettext('Failed to update the Asset'));
        }
        return Promise.reject(error);
    });
}
exports.updateAsset = updateAsset;
function lockAsset(original, updates) {
    var gettext = apis_1.superdeskApi.localization.gettext;
    var notify = apis_1.superdeskApi.ui.notify;
    return apis_1.superdeskApi.dataApi.patch(LOCK_ASSET, original, updates)
        .then(function (asset) {
        return asset;
    })
        .catch(function (error) {
        if (api_1.isSamsApiError(error)) {
            notify.error(api_1.getApiErrorMessage(error));
        }
        else {
            notify.error(gettext('Failed to lock the Asset'));
        }
        return Promise.reject(error);
    });
}
exports.lockAsset = lockAsset;
function unlockAsset(original, updates) {
    var gettext = apis_1.superdeskApi.localization.gettext;
    var notify = apis_1.superdeskApi.ui.notify;
    return apis_1.superdeskApi.dataApi.patch(UNLOCK_ASSET, original, updates)
        .then(function (asset) {
        return asset;
    })
        .catch(function (error) {
        if (api_1.isSamsApiError(error)) {
            notify.error(api_1.getApiErrorMessage(error));
        }
        else {
            notify.error(gettext('Failed to unlock the Asset'));
        }
        return Promise.reject(error);
    });
}
exports.unlockAsset = unlockAsset;
function getSetsSync() {
    var store = store_1.getStore();
    if (store === undefined) {
        throw new Error('SAMS store has not been initialised');
    }
    return selectors_1.getSetsById(store.getState());
}
exports.getSetsSync = getSetsSync;
