"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.deleteSet = exports.updateSet = exports.createSet = exports.getAllSets = void 0;
var apis_1 = require("../apis");
// Utils
var common_1 = require("./common");
var api_1 = require("../utils/api");
var RESOURCE = 'sams/sets';
function getAllSets() {
    var gettext = apis_1.superdeskApi.localization.gettext;
    var notify = apis_1.superdeskApi.ui.notify;
    return apis_1.superdeskApi.dataApi.query(RESOURCE, 1, { field: 'name', direction: 'ascending' }, {})
        .then(common_1.fixItemResponseVersionDates)
        .then(function (response) {
        var _a;
        return (_a = response === null || response === void 0 ? void 0 : response._items) !== null && _a !== void 0 ? _a : [];
    })
        .catch(function (error) {
        if (api_1.isSamsApiError(error)) {
            notify.error(api_1.getApiErrorMessage(error));
        }
        else {
            notify.error(gettext('Failed to load all sets'));
        }
        return Promise.reject(error);
    });
}
exports.getAllSets = getAllSets;
function createSet(item) {
    var gettext = apis_1.superdeskApi.localization.gettext;
    var notify = apis_1.superdeskApi.ui.notify;
    return apis_1.superdeskApi.dataApi.create(RESOURCE, item)
        .then(function (set) {
        notify.success(gettext('Set created successfully'));
        return set;
    })
        .catch(function (error) {
        if (api_1.isSamsApiError(error)) {
            notify.error(api_1.getApiErrorMessage(error));
        }
        else {
            notify.error(gettext('Failed to create the Set'));
        }
        return Promise.reject(error);
    });
}
exports.createSet = createSet;
function updateSet(original, updates) {
    var gettext = apis_1.superdeskApi.localization.gettext;
    var notify = apis_1.superdeskApi.ui.notify;
    return apis_1.superdeskApi.dataApi.patch(RESOURCE, original, updates)
        .then(function (set) {
        notify.success(gettext('Set updated successfully'));
        return set;
    })
        .catch(function (error) {
        if (api_1.isSamsApiError(error)) {
            notify.error(api_1.getApiErrorMessage(error));
        }
        else {
            notify.error(gettext('Failed to update the Set'));
        }
        return Promise.reject(error);
    });
}
exports.updateSet = updateSet;
function deleteSet(item) {
    var gettext = apis_1.superdeskApi.localization.gettext;
    var notify = apis_1.superdeskApi.ui.notify;
    return apis_1.superdeskApi.dataApi.delete(RESOURCE, item)
        .then(function () {
        notify.success(gettext('Set deleted successfully'));
    })
        .catch(function (error) {
        if (api_1.isSamsApiError(error)) {
            notify.error(api_1.getApiErrorMessage(error));
        }
        else {
            notify.error(gettext('Failed to delete the Set'));
        }
        return Promise.reject(error);
    });
}
exports.deleteSet = deleteSet;
