"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.deregisterWebsocketListeners = exports.registerWebsocketNotifications = void 0;
// Types
var apis_1 = require("../apis");
var sets_1 = require("./sets");
var assets_1 = require("./assets");
var websocketNotificationMap = {
    'sams:set:created': sets_1.onSetCreated,
    'sams:set:updated': sets_1.onSetUpdated,
    'sams:set:deleted': sets_1.onSetDeleted,
    'sams:asset:created': assets_1.onAssetCreated,
    'sams:asset:updated': assets_1.onAssetUpdated,
    'sams:asset:deleted': assets_1.onAssetDeleted,
    'sams:asset:lock_asset': assets_1.onAssetLocked,
    'sams:asset:unlock_asset': assets_1.onAssetUnlocked,
    'sams:asset:session_unlock': assets_1.onSessionUnlocked,
};
var websocketDeregistrationArray = [];
function registerWebsocketNotifications() {
    // To make sure that callbacks don't get registered multiple times.
    if (websocketDeregistrationArray.length > 0) {
        return;
    }
    Object.keys(websocketNotificationMap).forEach(function (name) {
        var deregisterListener = apis_1.superdeskApi.addWebsocketMessageListener(name, websocketNotificationMap[name]);
        websocketDeregistrationArray.push(deregisterListener);
    });
}
exports.registerWebsocketNotifications = registerWebsocketNotifications;
function deregisterWebsocketListeners() {
    websocketDeregistrationArray.forEach(function (deregisterListener) {
        deregisterListener();
    });
    websocketDeregistrationArray = [];
}
exports.deregisterWebsocketListeners = deregisterWebsocketListeners;
