"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.reloadAssetList = exports.loadAssetsByIds = exports.deleteAssets = exports.deleteAsset = exports.updateAsset = exports.onEditAsset = exports.lockAsset = exports.forceUnlockAsset = exports.unlockAsset = exports.editAsset = exports.updateAssetInStore = exports.updateAssetSearchParamsAndListItemsFromURL = exports.updateAssetSearchParamsAndListItems = exports.loadNextAssetsPage = exports.queryAssetsFromCurrentSearch = exports.queryAssets = exports.closeMultiActionBar = exports.updateSelectedAssetIds = exports.closeAssetContentPanel = exports.previewAsset = exports.toggleAssetListStyle = exports.setAssetListStyle = exports.sessionUnlock = exports.setAssetSearchParams = exports.receiveAssets = void 0;
var interfaces_1 = require("../../interfaces");
var types_1 = require("./types");
var apis_1 = require("../../apis");
// Redux Selectors
var selectors_1 = require("./selectors");
var selectors_2 = require("../sets/selectors");
// Utils
var assets_1 = require("../../utils/assets");
function receiveAssets(response, listAction) {
    return {
        type: types_1.RECEIVE_ASSETS,
        payload: {
            response: response,
            listAction: listAction,
        },
    };
}
exports.receiveAssets = receiveAssets;
function setAssetSearchParams(params) {
    return {
        type: types_1.SET_ASSET_SEARCH_PARAMS,
        payload: params,
    };
}
exports.setAssetSearchParams = setAssetSearchParams;
function sessionUnlock(session_id) {
    return function (dispatch, getState) {
        var assetItemIds = selectors_1.getAssetListItemIds(getState());
        var assets = selectors_1.getAssets(getState());
        var assetIds = [];
        Object.keys(assetItemIds).forEach(function (element) {
            if (assets[assetItemIds[element]].lock_session === session_id) {
                assetIds.push(assets[assetItemIds[element]]._id);
            }
        });
        return dispatch(loadAssetsByIds(assetIds));
    };
}
exports.sessionUnlock = sessionUnlock;
function setAssetListStyle(style) {
    return {
        type: types_1.ASSET_SET_LIST_STYLE,
        payload: style,
    };
}
exports.setAssetListStyle = setAssetListStyle;
function toggleAssetListStyle() {
    return function (dispatch, getState) {
        dispatch(setAssetListStyle(selectors_1.getAssetListStyle(getState()) === interfaces_1.ASSET_LIST_STYLE.GRID ?
            interfaces_1.ASSET_LIST_STYLE.LIST :
            interfaces_1.ASSET_LIST_STYLE.GRID));
        return dispatch(queryAssetsFromCurrentSearch());
    };
}
exports.toggleAssetListStyle = toggleAssetListStyle;
function previewAsset(asset_id) {
    return {
        type: types_1.MANAGE_ASSETS_PREVIEW,
        payload: asset_id,
    };
}
exports.previewAsset = previewAsset;
function closeAssetContentPanel() {
    return {
        type: types_1.MANAGE_ASSETS_CLOSE_CONTENT_PANEL,
    };
}
exports.closeAssetContentPanel = closeAssetContentPanel;
function updateSelectedAssetIds(asset_id) {
    return {
        type: types_1.UPDATE_SELECTED_ASSET_IDS,
        payload: asset_id,
    };
}
exports.updateSelectedAssetIds = updateSelectedAssetIds;
function closeMultiActionBar() {
    return {
        type: types_1.MANAGE_MULTIACTIONBAR_CLOSE,
    };
}
exports.closeMultiActionBar = closeMultiActionBar;
function queryAssets(params, listAction) {
    return function (dispatch, getState) {
        return apis_1.samsApi.assets.query(params, selectors_1.getAssetListStyle(getState()))
            .then(function (response) {
            dispatch(receiveAssets(response, listAction));
        });
    };
}
exports.queryAssets = queryAssets;
function queryAssetsFromCurrentSearch(listAction) {
    return function (dispatch, getState) {
        var params = selectors_1.getAssetSearchParams(getState());
        return dispatch(queryAssets(params, listAction));
    };
}
exports.queryAssetsFromCurrentSearch = queryAssetsFromCurrentSearch;
function loadNextAssetsPage() {
    return function (dispatch, getState) {
        var params = selectors_1.getAssetSearchParams(getState());
        params.page += 1;
        return dispatch(queryAssets(params, interfaces_1.LIST_ACTION.APPEND));
    };
}
exports.loadNextAssetsPage = loadNextAssetsPage;
function updateAssetSearchParamsAndListItems(params, listAction) {
    return function (dispatch, getState) {
        if (listAction === interfaces_1.LIST_ACTION.REPLACE) {
            dispatch(setAssetSearchParams(__assign(__assign({}, params), { page: 1 })));
        }
        else {
            dispatch(setAssetSearchParams(params));
        }
        apis_1.samsApi.assets.setSearchUrlParams(selectors_1.getAssetSearchParams(getState()));
        return dispatch(queryAssetsFromCurrentSearch(listAction));
    };
}
exports.updateAssetSearchParamsAndListItems = updateAssetSearchParamsAndListItems;
function updateAssetSearchParamsAndListItemsFromURL(listAction) {
    return function (dispatch, _getState) {
        var searchParams = apis_1.samsApi.assets.getSearchUrlParams();
        if (Object.keys(searchParams).length > 0) {
            dispatch(setAssetSearchParams(searchParams));
        }
        return dispatch(queryAssetsFromCurrentSearch(listAction));
    };
}
exports.updateAssetSearchParamsAndListItemsFromURL = updateAssetSearchParamsAndListItemsFromURL;
function updateAssetInStore(asset, assetId) {
    return {
        type: types_1.MANAGE_ASSET_UPDATE_IN_STORE,
        payload: {
            asset: asset,
            assetId: assetId,
        },
    };
}
exports.updateAssetInStore = updateAssetInStore;
function editAsset(assetId) {
    return {
        type: types_1.MANAGE_ASSETS_EDIT,
        payload: assetId,
    };
}
exports.editAsset = editAsset;
function unlockAsset(asset) {
    return function (dispatch, getState) {
        return apis_1.samsApi.assets.unlockAsset(asset, {})
            .then(function (unlockedAsset) {
            dispatch(updateAssetInStore(unlockedAsset, asset._id));
            var getassets = selectors_1.getAssets(getState());
            return getassets[asset._id];
        });
    };
}
exports.unlockAsset = unlockAsset;
function forceUnlockAsset(asset) {
    return function (dispatch, getState) {
        return apis_1.samsApi.assets.unlockAsset(asset, { 'force': true })
            .then(function (unlockedAsset) {
            dispatch(updateAssetInStore(unlockedAsset, asset._id));
            var getassets = selectors_1.getAssets(getState());
            return getassets[asset._id];
        });
    };
}
exports.forceUnlockAsset = forceUnlockAsset;
function lockAsset(asset) {
    return function (dispatch, getState) {
        if (assets_1.verifyAssetBeforeLocking(asset, 'edit')) {
            return Promise.resolve();
        }
        else {
            return apis_1.samsApi.assets.lockAsset(asset, { 'lock_action': 'edit' })
                .then(function (lockedAsset) {
                dispatch(updateAssetInStore(lockedAsset, asset._id));
                var getassets = selectors_1.getAssets(getState());
                return getassets[asset._id];
            });
        }
    };
}
exports.lockAsset = lockAsset;
function onEditAsset(asset) {
    return function (dispatch, getState) {
        var disabledSetIds = selectors_2.getDisabledSetIds(getState());
        if (disabledSetIds.indexOf(asset.set_id) === -1) {
            dispatch(lockAsset(asset))
                .then(function () {
                dispatch(editAsset(asset._id));
            });
        }
    };
}
exports.onEditAsset = onEditAsset;
function updateAsset(original, updates) {
    return function (dispatch) {
        return apis_1.samsApi.assets.update(original, updates)
            .then(function (updatedAsset) {
            // Wait for the Assets to update before returning the updated Asset
            dispatch(updateAssetInStore(updatedAsset, updatedAsset._id));
            return updatedAsset;
        });
    };
}
exports.updateAsset = updateAsset;
function deleteAsset(asset) {
    return function (dispatch, getState) {
        var selectedAssetId = selectors_1.getSelectedAssetId(getState());
        var selectedAssetIds = selectors_1.getSelectedAssetIds(getState());
        return apis_1.samsApi.assets.deleteAsset(asset)
            .then(function () {
            if (selectedAssetId === asset._id) {
                dispatch(closeAssetContentPanel());
            }
            if (selectedAssetIds.indexOf(asset._id) !== -1) {
                dispatch(updateSelectedAssetIds(asset._id));
            }
        });
    };
}
exports.deleteAsset = deleteAsset;
function deleteAssets(asset) {
    return function (dispatch, getState) {
        if (asset !== undefined) {
            return openDeleteConfirmationModal(asset.name)
                .then(function (response) {
                if (response === true) {
                    dispatch(deleteAsset(asset));
                }
            });
        }
        else {
            var selectedAssets_1 = selectors_1.getSelectedAssetItems(getState());
            var assetName = (selectedAssets_1.length === 1) ? selectedAssets_1[0].name : undefined;
            return openDeleteConfirmationModal(assetName, selectedAssets_1.length)
                .then(function (response) {
                if (response === true) {
                    Promise.all(selectedAssets_1.map(function (selectedAsset) { return dispatch(deleteAsset(selectedAsset)); })).finally(function () {
                        dispatch(closeMultiActionBar());
                    });
                }
                return Promise.resolve();
            });
        }
    };
}
exports.deleteAssets = deleteAssets;
function loadAssetsByIds(ids) {
    return function (dispatch, getState) {
        var loadedAssetIds = Object.keys(selectors_1.getAssets(getState()));
        var attachmentsToLoad = ids.filter(function (id) { return loadedAssetIds.includes(id); });
        if (attachmentsToLoad.length === 0) {
            return Promise.resolve();
        }
        return apis_1.samsApi.assets.getByIds(attachmentsToLoad)
            .then(function (response) {
            dispatch(receiveAssets(response));
        });
    };
}
exports.loadAssetsByIds = loadAssetsByIds;
function reloadAssetList() {
    return function (dispatch) { return (dispatch(updateAssetSearchParamsAndListItems({}, interfaces_1.LIST_ACTION.REPLACE))); };
}
exports.reloadAssetList = reloadAssetList;
function openDeleteConfirmationModal(assetName, asset_length) {
    var gettext = apis_1.superdeskApi.localization.gettext;
    var confirm = apis_1.superdeskApi.ui.confirm;
    var el = document.createElement('div');
    // FIXME: Add an extra backdrop that will cover the Manage Assets modal
    // This is required because the ui-framework calculates z-index
    // based on the number of active modals, where as we're using
    // a mixture of the ui-framework and pure React modals
    // (superdeskApi.ui.showModal vs superdeskApi.ui.confirm)
    el.classList.add('modal__backdrop', 'fade', 'in');
    el.style.zIndex = '1050';
    document.body.append(el);
    if (assetName !== undefined) {
        return confirm(gettext('Are you sure you want to delete the asset "{{name}}"?', { name: assetName }), gettext('Delete Asset?'))
            .then(function (response) {
            el.remove();
            return response;
        });
    }
    else if (asset_length !== undefined) {
        return confirm(gettext('Are you sure you want to delete these {{length}} assets', { length: asset_length }), gettext('Delete Asset?'))
            .then(function (response) {
            el.remove();
            return response;
        });
    }
    else {
        return Promise.reject();
    }
}
