"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.assetsReducer = void 0;
var interfaces_1 = require("../../interfaces");
var types_1 = require("./types");
var initialState = {
    assets: {},
    searchParams: {
        page: 1,
        mimetypes: interfaces_1.ASSET_TYPE_FILTER.ALL,
        sortField: interfaces_1.ASSET_SORT_FIELD.NAME,
        sortOrder: interfaces_1.SORT_ORDER.ASCENDING,
    },
    listItemIds: [],
    searchResultTotal: 0,
    contentPanelState: interfaces_1.ASSET_CONTENT_PANEL_STATE.CLOSED,
    listStyle: interfaces_1.ASSET_LIST_STYLE.GRID,
    selectedAssetId: undefined,
    selectedAssetIds: [],
};
function assetsReducer(state, action) {
    if (state === void 0) { state = initialState; }
    switch (action.type) {
        case types_1.RECEIVE_ASSETS:
            return receiveAssets(state, action.payload);
        case types_1.SET_ASSET_SEARCH_PARAMS:
            return updateSearchParams(state, action.payload);
        case types_1.MANAGE_ASSETS_PREVIEW:
            return __assign(__assign({}, state), { contentPanelState: interfaces_1.ASSET_CONTENT_PANEL_STATE.PREVIEW, selectedAssetId: action.payload });
        case types_1.MANAGE_ASSETS_CLOSE_CONTENT_PANEL:
            return __assign(__assign({}, state), { contentPanelState: interfaces_1.ASSET_CONTENT_PANEL_STATE.CLOSED, selectedAssetId: undefined });
        case types_1.UPDATE_SELECTED_ASSET_IDS:
            return manageAssetsInSelectedAssetsArray(state, action.payload);
        case types_1.MANAGE_MULTIACTIONBAR_CLOSE:
            return __assign(__assign({}, state), { selectedAssetIds: [] });
        case types_1.ASSET_SET_LIST_STYLE:
            return __assign(__assign({}, state), { listStyle: action.payload });
        case types_1.MANAGE_ASSETS_EDIT:
            return __assign(__assign({}, state), { contentPanelState: interfaces_1.ASSET_CONTENT_PANEL_STATE.EDIT, selectedAssetId: action.payload });
        case types_1.MANAGE_ASSET_UPDATE_IN_STORE:
            return updateAssetInStore(state, action.payload);
        default:
            return state;
    }
}
exports.assetsReducer = assetsReducer;
function receiveAssets(prevState, payload) {
    var newState = __assign({}, prevState);
    var assets = payload.response._items.reduce(function (items, asset) {
        items[asset._id] = asset;
        return items;
    }, {});
    newState.assets = __assign(__assign({}, prevState.assets), assets);
    if (payload.listAction != null) {
        var assetIds = payload.response._items.map(function (asset) { return asset._id; });
        if (payload.listAction === interfaces_1.LIST_ACTION.APPEND) {
            newState.listItemIds = newState.listItemIds.concat(assetIds);
        }
        else if (payload.listAction === interfaces_1.LIST_ACTION.REPLACE) {
            newState.listItemIds = assetIds;
        }
        newState.searchResultTotal = payload.response._meta.total;
    }
    return newState;
}
function updateSearchParams(prevState, params) {
    var _a, _b, _c, _d, _e, _f, _g, _h;
    var newState = __assign({}, prevState);
    newState.searchParams = __assign(__assign(__assign({}, prevState.searchParams), params), { page: (_b = (_a = params.page) !== null && _a !== void 0 ? _a : prevState.searchParams.page) !== null && _b !== void 0 ? _b : 1, mimetypes: (_d = (_c = params.mimetypes) !== null && _c !== void 0 ? _c : prevState.searchParams.mimetypes) !== null && _d !== void 0 ? _d : interfaces_1.ASSET_TYPE_FILTER.ALL, sortField: (_f = (_e = params.sortField) !== null && _e !== void 0 ? _e : prevState.searchParams.sortField) !== null && _f !== void 0 ? _f : interfaces_1.ASSET_SORT_FIELD.NAME, sortOrder: (_h = (_g = params.sortOrder) !== null && _g !== void 0 ? _g : prevState.searchParams.sortOrder) !== null && _h !== void 0 ? _h : interfaces_1.SORT_ORDER.ASCENDING });
    return newState;
}
function manageAssetsInSelectedAssetsArray(prevState, payload) {
    var newState = __assign({}, prevState);
    var selectedAssetId = payload;
    var selectedAssetIdsArray = __spreadArrays(prevState.selectedAssetIds);
    if (selectedAssetIdsArray.indexOf(selectedAssetId) === -1) {
        newState.selectedAssetIds = selectedAssetIdsArray.concat([selectedAssetId]);
    }
    else {
        selectedAssetIdsArray.splice(selectedAssetIdsArray.indexOf(selectedAssetId), 1);
        newState.selectedAssetIds = selectedAssetIdsArray;
    }
    return newState;
}
function updateAssetInStore(prevState, payload) {
    var assets = __assign({}, prevState.assets);
    var updates = payload.asset;
    var assetId = payload.assetId;
    assets[assetId] = __assign(__assign({}, prevState.assets[assetId]), updates);
    return __assign(__assign({}, prevState), { assets: assets });
}
