"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.unsetStore = exports.getStore = exports.getStoreSingleton = exports.createReduxStore = exports.rootReducer = void 0;
// External Modules
var redux_1 = require("redux");
var redux_thunk_1 = __importDefault(require("redux-thunk"));
var redux_logger_1 = require("redux-logger");
// Redux Reducers
var reducers_1 = require("./sets/reducers");
var reducers_2 = require("./storageDestinations/reducers");
var reducers_3 = require("./assets/reducers");
var reducers_4 = require("./workspace/reducers");
var notifications_1 = require("../notifications");
exports.rootReducer = redux_1.combineReducers({
    sets: reducers_1.setsReducer,
    storageDestinations: reducers_2.storageDestinationReducer,
    assets: reducers_3.assetsReducer,
    workspace: reducers_4.workspaceReducer,
});
/**
 * Some action dispatchers (specifically thunk with promises)
 * do not catch javascript exceptions.
 * This middleware ensures that uncaught exceptions are still thrown
 * displaying the error in the console.
 */
function crashReporter() {
    return function (next) {
        return function (action) {
            // eslint-disable-next-line no-useless-catch
            try {
                return next(action);
            }
            catch (error) {
                throw error;
            }
        };
    };
}
var store;
var storeReferenceCount = 0;
function createReduxStore(initialState, reducer) {
    var middlewares = [
        crashReporter,
        redux_thunk_1.default,
    ];
    if (process.env.NODE_ENV !== 'production') {
        // activate logs actions for non production instances.
        // (this should always be the last middleware)
        middlewares.push(redux_logger_1.createLogger());
    }
    notifications_1.registerWebsocketNotifications();
    store = redux_1.createStore(reducer, initialState, redux_1.compose(redux_1.applyMiddleware.apply(void 0, middlewares)));
    return store;
}
exports.createReduxStore = createReduxStore;
function getStoreSingleton() {
    if (store === undefined) {
        store = createReduxStore({}, exports.rootReducer);
    }
    storeReferenceCount += 1;
    return store;
}
exports.getStoreSingleton = getStoreSingleton;
function getStore() {
    return store;
}
exports.getStore = getStore;
function unsetStore() {
    storeReferenceCount -= 1;
    if (storeReferenceCount === 0) {
        notifications_1.deregisterWebsocketListeners();
        store = undefined;
    }
}
exports.unsetStore = unsetStore;
