"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.confirmBeforeDeletingSet = exports.loadAssetsCount = exports.loadSets = exports.receiveAssetsCount = exports.onManageSetsModalClosed = exports.closeSetContentPanel = exports.previewSet = exports.editSet = exports.removeSetInStore = exports.receiveSets = void 0;
var apis_1 = require("../../apis");
// Redux Selectors
var selectors_1 = require("./selectors");
var types_1 = require("./types");
function receiveSets(sets) {
    return {
        type: types_1.RECEIVE_SETS,
        payload: sets,
    };
}
exports.receiveSets = receiveSets;
function removeSetInStore(set) {
    return {
        type: types_1.REMOVE_SET_IN_STORE,
        payload: set,
    };
}
exports.removeSetInStore = removeSetInStore;
function editSet(setId) {
    return {
        type: types_1.MANAGE_SETS_EDIT,
        payload: setId,
    };
}
exports.editSet = editSet;
function previewSet(setId) {
    return {
        type: types_1.MANAGE_SETS_PREVIEW,
        payload: setId,
    };
}
exports.previewSet = previewSet;
function closeSetContentPanel() {
    return {
        type: types_1.MANAGE_SETS_CLOSE_CONTENT_PANEL,
    };
}
exports.closeSetContentPanel = closeSetContentPanel;
function onManageSetsModalClosed() {
    return {
        type: types_1.MANAGE_SETS_RESET,
    };
}
exports.onManageSetsModalClosed = onManageSetsModalClosed;
function receiveAssetsCount(counts) {
    return {
        type: types_1.RECEIVE_ASSETS_COUNT,
        payload: counts,
    };
}
exports.receiveAssetsCount = receiveAssetsCount;
function loadSets() {
    return function (dispatch) {
        return apis_1.samsApi.sets.getAll()
            .then(function (sets) {
            var setIds = sets.map(function (set) {
                return set._id;
            });
            dispatch(receiveSets(sets));
            return dispatch(loadAssetsCount(setIds))
                .then(function () { return sets; });
        });
    };
}
exports.loadSets = loadSets;
function loadAssetsCount(setIds) {
    return function (dispatch) {
        return apis_1.samsApi.assets.getCount(setIds)
            .then(function (counts) {
            dispatch(receiveAssetsCount(counts));
            return Promise.resolve(counts);
        });
    };
}
exports.loadAssetsCount = loadAssetsCount;
function openDeleteConfirmationModal(set) {
    var gettext = apis_1.superdeskApi.localization.gettext;
    var confirm = apis_1.superdeskApi.ui.confirm;
    var el = document.createElement('div');
    // FIXME: Add an extra backdrop that will cover the Manage Sets modal
    // This is required because the ui-framework calculates z-index
    // based on the number of active modals, where as we're using
    // a mixture of the ui-framework and pure React modals
    // (superdeskApi.ui.showModal vs superdeskApi.ui.confirm)
    el.classList.add('modal__backdrop', 'fade', 'in');
    el.style.zIndex = '1050';
    document.body.append(el);
    return confirm(gettext('Are you sure you want to delete the Set "{{name}}"?', { name: set.name }), gettext('Delete Set?'))
        .then(function (response) {
        el.remove();
        return response;
    });
}
function confirmBeforeDeletingSet(set) {
    return function (dispatch, getState) {
        return openDeleteConfirmationModal(set)
            .then(function (response) {
            if (response === true) {
                return apis_1.samsApi.sets.delete(set)
                    .then(function () {
                    if (selectors_1.getSelectedSetId(getState()) === set._id) {
                        dispatch(closeSetContentPanel());
                    }
                });
            }
            return Promise.resolve();
        });
    };
}
exports.confirmBeforeDeletingSet = confirmBeforeDeletingSet;
