"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getSelectedSetCount = exports.getAssetsCountForSets = exports.getSetsGroupedByState = exports.getSelectedSetStorageDestination = exports.getSelectedSet = exports.getSelectedSetId = exports.getSetContentPanelState = exports.getSetsById = exports.getDisabledSetIds = exports.getDisabledSets = exports.getActiveSets = exports.getSets = void 0;
// External Modules
var reselect_1 = require("reselect");
var interfaces_1 = require("../../interfaces");
// Redux Selectors
var selectors_1 = require("../storageDestinations/selectors");
// Utils
var typescript_1 = require("../../utils/typescript");
function getSets(state) {
    return state.sets.sets;
}
exports.getSets = getSets;
function getActiveSets(state) {
    return state.sets.sets.filter(function (set) { return set.state === interfaces_1.SET_STATE.USABLE; });
}
exports.getActiveSets = getActiveSets;
function getDisabledSets(state) {
    return state.sets.sets.filter(function (set) { return set.state === interfaces_1.SET_STATE.DISABLED; });
}
exports.getDisabledSets = getDisabledSets;
function getDisabledSetIds(state) {
    var disabledSets = getDisabledSets(state);
    var disabledSetIds = [];
    disabledSets.forEach(function (set) {
        disabledSetIds.push(set._id);
    });
    return disabledSetIds;
}
exports.getDisabledSetIds = getDisabledSetIds;
exports.getSetsById = reselect_1.createSelector([getSets], function (sets) {
    return sets.reduce(function (items, set) {
        items[set._id] = set;
        return items;
    }, {});
});
function getSetContentPanelState(state) {
    var _a;
    return (_a = state.sets.contentPanelState) !== null && _a !== void 0 ? _a : interfaces_1.CONTENT_PANEL_STATE.CLOSED;
}
exports.getSetContentPanelState = getSetContentPanelState;
function getSelectedSetId(state) {
    return state.sets.selectedSetId;
}
exports.getSelectedSetId = getSelectedSetId;
exports.getSelectedSet = reselect_1.createSelector([exports.getSetsById, getSelectedSetId], function (sets, setId) { return (setId != null ? sets === null || sets === void 0 ? void 0 : sets[setId] :
    undefined); });
exports.getSelectedSetStorageDestination = reselect_1.createSelector([exports.getSelectedSet, selectors_1.getStorageDestinationsById], function (set, destinations) { return ((set === null || set === void 0 ? void 0 : set.destination_name) != null ? destinations === null || destinations === void 0 ? void 0 : destinations[set === null || set === void 0 ? void 0 : set.destination_name] :
    undefined); });
exports.getSetsGroupedByState = reselect_1.createSelector([getSets], function (sets) {
    var groupedSets = {
        draft: [],
        usable: [],
        disabled: [],
    };
    sets.forEach(function (set) {
        switch (set.state) {
            case interfaces_1.SET_STATE.USABLE:
                groupedSets.usable.push(set);
                break;
            case interfaces_1.SET_STATE.DISABLED:
                groupedSets.disabled.push(set);
                break;
            case interfaces_1.SET_STATE.DRAFT:
                groupedSets.draft.push(set);
                break;
            default:
                typescript_1.assertNever(set.state);
        }
    });
    return groupedSets;
});
function getAssetsCountForSets(state) {
    return state.sets.counts;
}
exports.getAssetsCountForSets = getAssetsCountForSets;
exports.getSelectedSetCount = reselect_1.createSelector([getAssetsCountForSets, getSelectedSetId], function (counts, setId) { return (setId != null ? counts === null || counts === void 0 ? void 0 : counts[setId] :
    undefined); });
