"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getApiErrorMessage = exports.API_ERRORS = exports.isSamsApiError = exports.hasItemChanged = exports.generatePatch = void 0;
// External Modules
var json_merge_patch_1 = require("json-merge-patch");
var apis_1 = require("../apis");
function generatePatch(original, updates) {
    var _a;
    return ((_a = json_merge_patch_1.generate(original, updates)) !== null && _a !== void 0 ? _a : {});
}
exports.generatePatch = generatePatch;
function hasItemChanged(original, updates) {
    return Object.keys(generatePatch(original, updates)).length > 0;
}
exports.hasItemChanged = hasItemChanged;
function isSamsApiError(error) {
    return (error === null || error === void 0 ? void 0 : error.error) != null &&
        (error === null || error === void 0 ? void 0 : error.name) != null &&
        (error === null || error === void 0 ? void 0 : error.description) != null;
}
exports.isSamsApiError = isSamsApiError;
// Provide translations here, as the SAMS API does not currently support localisation
exports.API_ERRORS = {
    '04002': function (_error) {
        return apis_1.superdeskApi.localization.gettext('Error[04002]: Invalid search query');
    },
    '04001': function (_error) {
        for (var _i = 0, _a = Object.keys(_error.errors); _i < _a.length; _i++) {
            var error = _a[_i];
            for (var _b = 0, _c = _error.errors[error]; _b < _c.length; _b++) {
                var key = _c[_b];
                if (key === 'unique') {
                    return apis_1.superdeskApi.localization.gettext('Error[{{number}}]: {{error}} not unique', {
                        number: _error.error,
                        error: error.toUpperCase(),
                    });
                }
                else if (key === 'required' || key === 'empty values not allowed') {
                    return apis_1.superdeskApi.localization.gettext('Error[{{number}}]: {{error}} requried', {
                        number: _error.error,
                        error: error.toUpperCase(),
                    });
                }
            }
        }
        return apis_1.superdeskApi.localization.gettext('Error[{{number}}]: {{description}}', {
            number: _error.error,
            description: _error.description,
        });
    },
};
function getApiErrorMessage(error) {
    if (exports.API_ERRORS[error.error] != null) {
        return exports.API_ERRORS[error.error](error);
    }
    return apis_1.superdeskApi.localization.gettext('Error[{{number}}]: {{description}}', {
        number: error.error,
        description: error.description,
    });
}
exports.getApiErrorMessage = getApiErrorMessage;
