"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isAssetLockedByCurrentUser = exports.isSetDisabled = exports.isAssetLocked = exports.verifyAssetBeforeLocking = exports.getMimetypeHumanReadable = exports.getDropdownItemsForActions = exports.getActions = exports.getAction = void 0;
var interfaces_1 = require("../interfaces");
var apis_1 = require("../apis");
var assets_1 = require("../api/assets");
function getAction(assetCallback) {
    var gettext = apis_1.superdeskApi.localization.gettext;
    var assertNever = apis_1.superdeskApi.helpers.assertNever;
    switch (assetCallback.action) {
        case interfaces_1.ASSET_ACTIONS.PREVIEW:
            return {
                id: interfaces_1.ASSET_ACTIONS.PREVIEW,
                label: gettext('Preview'),
                icon: 'eye-open',
                onSelect: assetCallback.onSelect,
                isAllowed: function (asset) { return asset._created != null; },
            };
        case interfaces_1.ASSET_ACTIONS.DOWNLOAD:
            return {
                id: interfaces_1.ASSET_ACTIONS.DOWNLOAD,
                label: gettext('Download'),
                icon: 'download',
                onSelect: assetCallback.onSelect,
                isAllowed: function (asset) { return asset._created != null; },
            };
        case interfaces_1.ASSET_ACTIONS.DELETE:
            return {
                id: interfaces_1.ASSET_ACTIONS.DELETE,
                label: gettext('Delete'),
                icon: 'trash',
                onSelect: assetCallback.onSelect,
                isAllowed: function (asset) { return asset._created != null && isAssetLockedByCurrentUser(asset); },
            };
        case interfaces_1.ASSET_ACTIONS.EDIT:
            return {
                id: interfaces_1.ASSET_ACTIONS.EDIT,
                label: gettext('Edit'),
                icon: 'pencil',
                onSelect: assetCallback.onSelect,
                isAllowed: function (asset) { return asset._created != null && isSetDisabled(asset) && isAssetLockedByCurrentUser(asset); },
            };
        case interfaces_1.ASSET_ACTIONS.FORCE_UNLOCK:
            return {
                id: interfaces_1.ASSET_ACTIONS.FORCE_UNLOCK,
                label: gettext('Force Unlock'),
                icon: 'unlocked',
                onSelect: assetCallback.onSelect,
                isAllowed: function (asset) { return asset._created != null && isAssetLocked(asset); },
            };
    }
    assertNever(assetCallback.action);
}
exports.getAction = getAction;
function getActions(asset, actions) {
    return actions == null || actions.length === 0 ?
        [] :
        actions
            .map(function (actionCallback) { return getAction(actionCallback); })
            .filter(function (action) { return action.isAllowed(asset); });
}
exports.getActions = getActions;
function getDropdownItemsForActions(asset, requestedActions) {
    var allowedActions = getActions(asset, requestedActions);
    if (allowedActions.length > 0) {
        return allowedActions.map(function (action) { return ({
            label: action.label,
            icon: action.icon,
            onSelect: function () { return action.onSelect(asset); },
        }); });
    }
    return [];
}
exports.getDropdownItemsForActions = getDropdownItemsForActions;
function getMimetypeHumanReadable(mimetype) {
    var gettext = apis_1.superdeskApi.localization.gettext;
    if (mimetype == null || mimetype.length === 0) {
        return '';
    }
    else if (mimetype.startsWith('image/')) {
        return gettext('Image');
    }
    else if (mimetype.startsWith('video/')) {
        return gettext('Video');
    }
    else if (mimetype.startsWith('audio/')) {
        return gettext('Audio');
    }
    else if (mimetype === 'application/pdf') {
        return gettext('PDF');
    }
    else if (mimetype === 'text/plain') {
        return gettext('Text');
    }
    else if (mimetype.includes('document')) {
        return gettext('Document');
    }
    else {
        return gettext('Other');
    }
}
exports.getMimetypeHumanReadable = getMimetypeHumanReadable;
function verifyAssetBeforeLocking(asset, lock_action) {
    var session_id = apis_1.superdeskApi.session.getSessionId();
    var user_id = apis_1.superdeskApi.session.getCurrentUserId();
    if (asset.lock_action === lock_action && asset.lock_session === session_id && asset.lock_user === user_id) {
        return true;
    }
    else {
        return false;
    }
}
exports.verifyAssetBeforeLocking = verifyAssetBeforeLocking;
function isAssetLocked(asset) {
    if (asset.lock_session != null) {
        return true;
    }
    else {
        return false;
    }
}
exports.isAssetLocked = isAssetLocked;
function isSetDisabled(asset) {
    var sets = assets_1.getSetsSync();
    if (sets[asset.set_id].state === 'disabled') {
        return false;
    }
    else {
        return true;
    }
}
exports.isSetDisabled = isSetDisabled;
function isAssetLockedByCurrentUser(asset) {
    var user_id = apis_1.superdeskApi.session.getCurrentUserId();
    if (asset.lock_user === user_id || !isAssetLocked(asset)) {
        return true;
    }
    else {
        return false;
    }
}
exports.isAssetLockedByCurrentUser = isAssetLockedByCurrentUser;
