"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAssetListSortFieldText = exports.getAssetStateLabel = exports.getFileSizeFromHumanReadable = exports.getIconTypeFromMimetype = exports.getHumanReadableFileSize = exports.showModalConnectedToStore = void 0;
// External Modules
var React = __importStar(require("react"));
var react_redux_1 = require("react-redux");
// Types
var interfaces_1 = require("../interfaces");
var apis_1 = require("../apis");
// Redux Actions & Selectors
var store_1 = require("../store");
// UI
var react_1 = require("superdesk-ui-framework/react");
function showModalConnectedToStore(Component, props) {
    var store = store_1.getStore();
    if (store == null) {
        return Promise.reject('SAMS store has not been initialised');
    }
    return apis_1.superdeskApi.ui.showModal(function (_a) {
        var closeModal = _a.closeModal;
        return (React.createElement(react_redux_1.Provider, { store: store },
            React.createElement(Component, __assign({ closeModal: closeModal }, props !== null && props !== void 0 ? props : {}))));
    });
}
exports.showModalConnectedToStore = showModalConnectedToStore;
function getHumanReadableFileSize(fileSize) {
    if (fileSize < 1024) {
        return fileSize + 'bytes';
    }
    else if (fileSize < 1048576) {
        return (fileSize / 1024).toFixed(1) + 'KB';
    }
    else {
        return (fileSize / 1048576).toFixed(1) + 'MB';
    }
}
exports.getHumanReadableFileSize = getHumanReadableFileSize;
function getIconTypeFromMimetype(mimetype) {
    if (mimetype.startsWith('image/')) {
        return 'photo';
    }
    else if (mimetype.startsWith('video/')) {
        return 'video';
    }
    else if (mimetype.startsWith('audio/')) {
        return 'audio';
    }
    else {
        return 'text';
    }
}
exports.getIconTypeFromMimetype = getIconTypeFromMimetype;
function getFileSizeFromHumanReadable(fileSize, dataUnit) {
    switch (dataUnit) {
        case interfaces_1.DATA_UNIT.BYTES:
            return fileSize;
        case interfaces_1.DATA_UNIT.KB:
            return fileSize * 1024;
        case interfaces_1.DATA_UNIT.MB:
            return fileSize * 1024 * 1024;
        case interfaces_1.DATA_UNIT.GB:
            return fileSize * 1024 * 1024 * 1024;
    }
}
exports.getFileSizeFromHumanReadable = getFileSizeFromHumanReadable;
function getAssetStateLabel(assetState) {
    var gettext = apis_1.superdeskApi.localization.gettext;
    switch (assetState) {
        case interfaces_1.ASSET_STATE.INTERNAL:
            return (React.createElement(react_1.Label, { text: gettext('Internal'), style: "hollow", color: "red--800" }));
        case interfaces_1.ASSET_STATE.DRAFT:
            return (React.createElement(react_1.Label, { text: gettext('Draft'), style: "hollow", type: "warning" }));
        case interfaces_1.ASSET_STATE.PUBLIC:
            return (React.createElement(react_1.Label, { text: gettext('Public'), style: "hollow", type: "success" }));
    }
    apis_1.superdeskApi.helpers.assertNever(assetState);
}
exports.getAssetStateLabel = getAssetStateLabel;
function getAssetListSortFieldText(field) {
    var gettext = apis_1.superdeskApi.localization.gettext;
    switch (field) {
        case interfaces_1.ASSET_SORT_FIELD.NAME:
            return gettext('Name');
        case interfaces_1.ASSET_SORT_FIELD.FILENAME:
            return gettext('Filename');
        case interfaces_1.ASSET_SORT_FIELD.SIZE:
            return gettext('Size');
        case interfaces_1.ASSET_SORT_FIELD.CREATED:
            return gettext('Created');
        case interfaces_1.ASSET_SORT_FIELD.UPDATED:
            return gettext('Updated');
    }
    apis_1.superdeskApi.helpers.assertNever(field);
}
exports.getAssetListSortFieldText = getAssetListSortFieldText;
