"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
class Dropdown extends React.Component {
    constructor(props) {
        super(props);
        this.state = {
            open: false,
            selectedItem: { label: '', value: 0 },
        };
        this.handleToggle = this.handleToggle.bind(this);
        this.handleSelect = this.handleSelect.bind(this);
    }
    componentDidUpdate(prevProps) {
        if (this.props.disabled !== prevProps.disabled) {
            this.handleDisable();
        }
    }
    handleDisable() {
        if (this.props.disabled === true) {
            this.setState({
                open: false,
                selectedItem: { label: '', value: 0 },
            });
        }
    }
    handleToggle() {
        // use for custom action when item is already selected
        // e.g. toggle crop mode
        if (!this.state.open && this.state.selectedItem.value && this.props.isButton) {
            this.props.onSelect(0);
            this.setState({ selectedItem: { label: '', value: 0 } });
            return;
        }
        this.setState({
            open: !this.state.open,
        });
    }
    handleSelect(item) {
        this.setState({ open: false, selectedItem: item });
        this.props.onSelect(item.value);
    }
    render() {
        const { gettext } = this.props;
        return (React.createElement("div", { className: `dropdown ${this.state.open ? 'open' : ''} ${this.props.className || ''}` },
            React.cloneElement(this.props.label, {
                onClick: this.handleToggle,
                selectedItem: this.state.selectedItem,
            }),
            React.createElement("ul", { className: "dropdown__menu" }, this.props.items.map((item) => (React.createElement("li", { key: item.value, onClick: () => this.handleSelect(item) },
                React.createElement("button", null, gettext(item.label))))))));
    }
}
exports.Dropdown = Dropdown;
